from typing import List, Tuple, Union

from solders.account import Account
from solders.instruction import CompiledInstruction
from solders.pubkey import Pubkey
from solders.signature import Signature
from solders.transaction_status import TransactionErrorType, TransactionReturnData

class InnerInstruction:
    def instruction(self) -> CompiledInstruction: ...
    @staticmethod
    def from_bytes(raw_bytes: bytes) -> "InnerInstruction": ...
    @staticmethod
    def from_json(raw: str) -> "InnerInstruction": ...
    def to_json(self) -> str: ...
    def __bytes__(self) -> bytes: ...
    def __str__(self) -> str: ...
    def __repr__(self) -> str: ...
    def __richcmp__(self, other: "InnerInstruction", op: int) -> bool: ...

class TransactionMetadata:
    def signature(self) -> Signature: ...
    def logs(self) -> List[str]: ...
    def inner_instructions(self) -> List[List[InnerInstruction]]: ...
    def compute_units_consumed(self) -> int: ...
    def transaction_return_data(self) -> TransactionReturnData: ...
    @staticmethod
    def from_bytes(raw_bytes: bytes) -> "TransactionMetadata": ...
    @staticmethod
    def from_json(raw: str) -> "TransactionMetadata": ...
    def to_json(self) -> str: ...
    def __bytes__(self) -> bytes: ...
    def __str__(self) -> str: ...
    def __repr__(self) -> str: ...
    def __richcmp__(self, other: "TransactionMetadata", op: int) -> bool: ...

class FailedTransactionMetadata:
    def err(self) -> TransactionErrorType: ...
    def meta(self) -> TransactionMetadata: ...
    @staticmethod
    def from_bytes(raw_bytes: bytes) -> "FailedTransactionMetadata": ...
    @staticmethod
    def from_json(raw: str) -> "FailedTransactionMetadata": ...
    def to_json(self) -> str: ...
    def __bytes__(self) -> bytes: ...
    def __str__(self) -> str: ...
    def __repr__(self) -> str: ...
    def __richcmp__(self, other: "FailedTransactionMetadata", op: int) -> bool: ...

class SimulatedTransactionInfo:
    def meta(self) -> TransactionMetadata: ...
    def post_accounts(self) -> List[Tuple[Pubkey, Account]]: ...
    @staticmethod
    def from_bytes(raw_bytes: bytes) -> "SimulatedTransactionInfo": ...
    @staticmethod
    def from_json(raw: str) -> "SimulatedTransactionInfo": ...
    def to_json(self) -> str: ...
    def __bytes__(self) -> bytes: ...
    def __str__(self) -> str: ...
    def __repr__(self) -> str: ...
    def __richcmp__(self, other: "SimulatedTransactionInfo", op: int) -> bool: ...

TransactionResult = Union[TransactionMetadata, FailedTransactionMetadata]
SimulateResult = Union[SimulatedTransactionInfo, FailedTransactionMetadata]



from solders.hash import Hash

class EpochRewards:
    def __init__(
        self,
        distribution_starting_block_height: int,
        num_partitions: int,
        parent_blockhash: Hash,
        total_points: int,
        total_rewards: int,
        distributed_rewards: int,
        active: bool,
    ) -> None: ...
    @property
    def distribution_starting_block_height(self) -> int: ...
    @distribution_starting_block_height.setter
    def distribution_starting_block_height(self, val: int) -> None: ...
    @property
    def num_partitions(self) -> int: ...
    @num_partitions.setter
    def num_partitions(self, val: int) -> None: ...
    @property
    def parent_blockhash(self) -> Hash: ...
    @parent_blockhash.setter
    def parent_blockhash(self, val: Hash) -> None: ...
    @property
    def total_points(self) -> int: ...
    @total_points.setter
    def total_points(self, val: int) -> None: ...
    @property
    def total_rewards(self) -> int: ...
    @total_rewards.setter
    def total_rewards(self, val: int) -> None: ...
    @property
    def distributed_rewards(self) -> int: ...
    @distributed_rewards.setter
    def distributed_rewards(self, val: int) -> None: ...
    @property
    def active(self) -> bool: ...
    @active.setter
    def active(self, val: bool) -> None: ...
    @staticmethod
    def from_bytes(raw_bytes: bytes) -> "EpochRewards": ...
    @staticmethod
    def from_json(raw: str) -> "EpochRewards": ...
    def to_json(self) -> str: ...
    def __bytes__(self) -> bytes: ...
    def __str__(self) -> str: ...
    def __repr__(self) -> str: ...
    def __richcmp__(self, other: "EpochRewards", op: int) -> bool: ...



from typing import Optional

class StakeHistoryEntry:
    def __init__(
        self,
        effective: int,
        activating: int,
        deactivating: int,
    ) -> None: ...
    @property
    def effective(self) -> int: ...
    @effective.setter
    def effective(self, val: int) -> None: ...
    @property
    def activating(self) -> int: ...
    @activating.setter
    def activating(self, val: int) -> None: ...
    @property
    def deactivating(self) -> int: ...
    @deactivating.setter
    def deactivating(self, val: int) -> None: ...
    @staticmethod
    def from_bytes(raw_bytes: bytes) -> "StakeHistoryEntry": ...
    @staticmethod
    def from_json(raw: str) -> "StakeHistoryEntry": ...
    def to_json(self) -> str: ...
    def __bytes__(self) -> bytes: ...
    def __str__(self) -> str: ...
    def __repr__(self) -> str: ...
    def __richcmp__(self, other: "StakeHistoryEntry", op: int) -> bool: ...

class StakeHistory:
    def __init__(
        self,
    ) -> None: ...
    def get(self, epoch: int) -> Optional[StakeHistoryEntry]: ...
    def add(self, epoch: int, entry: StakeHistoryEntry) -> None: ...
    @staticmethod
    def from_bytes(raw_bytes: bytes) -> "StakeHistory": ...
    @staticmethod
    def from_json(raw: str) -> "StakeHistory": ...
    def to_json(self) -> str: ...
    def __bytes__(self) -> bytes: ...
    def __str__(self) -> str: ...
    def __repr__(self) -> str: ...
    def __richcmp__(self, other: "StakeHistory", op: int) -> bool: ...



from pathlib import Path
from typing import List, Optional, Sequence, Tuple, Union

from solders.account import Account
from solders.clock import Clock
from solders.commitment_config import CommitmentLevel
from solders.hash import Hash
from solders.keypair import Keypair
from solders.message import Message
from solders.pubkey import Pubkey
from solders.rent import Rent
from solders.signature import Signature
from solders.transaction import Transaction, VersionedTransaction
from solders.transaction_status import (
    TransactionErrorType,
    TransactionReturnData,
    TransactionStatus,
)

class BanksClient:
    async def get_account(
        self, address: Pubkey, commitment: Optional[CommitmentLevel] = None
    ) -> Optional[Account]: ...
    async def get_balance(
        self,
        address: Pubkey,
        commitment: Optional[CommitmentLevel] = None,
    ) -> int: ...
    async def get_block_height(
        self, commitment: Optional[CommitmentLevel] = None
    ) -> int: ...
    async def get_clock(self) -> Clock: ...
    async def get_fee_for_message(
        self,
        message: Message,
        commitment: Optional[CommitmentLevel] = None,
    ) -> Optional[int]: ...
    async def get_latest_blockhash(
        self, commitment: Optional[CommitmentLevel] = None
    ) -> Tuple[Hash, int]: ...
    async def get_rent(self) -> Rent: ...
    async def get_slot(self, commitment: Optional[CommitmentLevel] = None) -> int: ...
    async def get_transaction_status(
        self, signature: Signature
    ) -> Optional[TransactionStatus]: ...
    async def get_transaction_statuses(
        self, signatures: Sequence[Signature]
    ) -> List[Optional[Signature]]: ...
    async def process_transaction(
        self,
        transaction: Union[Transaction, VersionedTransaction],
    ) -> BanksTransactionMeta: ...
    async def send_transaction(
        self, transaction: Union[Transaction, VersionedTransaction]
    ) -> None: ...
    async def simulate_transaction(
        self,
        transaction: Union[Transaction, VersionedTransaction],
        commitment: Optional[CommitmentLevel] = None,
    ) -> BanksTransactionResultWithMeta: ...

class BanksClientError(Exception): ...

class BanksTransactionMeta:
    @property
    def compute_units_consumed(self) -> int: ...
    @property
    def log_messages(self) -> List[str]: ...
    @property
    def return_data(self) -> Optional[TransactionReturnData]: ...
    def __init__(
        self,
        compute_units_consumed: int,
        log_messages: Sequence[str],
        return_data: Optional[TransactionReturnData] = None,
    ) -> None: ...
    @staticmethod
    def from_bytes(raw_bytes: bytes) -> "BanksTransactionMeta": ...
    @staticmethod
    def from_json(raw: str) -> "BanksTransactionMeta": ...
    def to_json(self) -> str: ...
    def __bytes__(self) -> bytes: ...
    def __str__(self) -> str: ...
    def __repr__(self) -> str: ...
    def __richcmp__(self, other: "BanksTransactionMeta", op: int) -> bool: ...

class BanksTransactionResultWithMeta:
    @property
    def meta(self) -> Optional[BanksTransactionMeta]: ...
    @property
    def result(self) -> Optional[TransactionErrorType]: ...
    def __init__(
        self,
        result: Optional[TransactionErrorType] = None,
        meta: Optional[BanksTransactionMeta] = None,
    ) -> None: ...
    @staticmethod
    def from_bytes(raw_bytes: bytes) -> "BanksTransactionResultWithMeta": ...
    @staticmethod
    def from_json(raw: str) -> "BanksTransactionResultWithMeta": ...
    def to_json(self) -> str: ...
    def __bytes__(self) -> bytes: ...
    def __str__(self) -> str: ...
    def __repr__(self) -> str: ...
    def __richcmp__(self, other: "BanksTransactionResultWithMeta", op: int) -> bool: ...

class ProgramTestContext:
    @property
    def banks_client(self) -> BanksClient: ...
    @property
    def last_blockhash(self) -> Hash: ...
    @property
    def payer(self) -> Keypair: ...
    def increment_vote_account_credits(
        self,
        vote_account_address: Pubkey,
        number_of_credits: int,
    ) -> None: ...
    def set_account(self, address: Pubkey, account: Account) -> None: ...
    def set_clock(self, clock: Clock) -> None: ...
    def set_rent(self, rent: Rent) -> None: ...
    def warp_to_slot(self, warp_slot: int) -> None: ...

async def start(
    programs: Optional[Sequence[Tuple[str, Pubkey]]] = None,
    accounts: Optional[Sequence[Tuple[Pubkey, Account]]] = None,
    compute_max_units: Optional[int] = None,
    transaction_account_lock_limit: Optional[int] = None,
) -> ProgramTestContext: ...
async def start_anchor(
    path: Path,
    extra_programs: Optional[Sequence[Tuple[str, Pubkey]]] = None,
    accounts: Optional[Sequence[Tuple[Pubkey, Account]]] = None,
    compute_max_units: Optional[int] = None,
    transaction_account_lock_limit: Optional[int] = None,
) -> ProgramTestContext: ...



DEFAULT_DEV_SLOTS_PER_EPOCH: int
DEFAULT_HASHES_PER_SECOND: int
DEFAULT_HASHES_PER_TICK: int
DEFAULT_MS_PER_SLOT: int
DEFAULT_SLOTS_PER_EPOCH: int
DEFAULT_S_PER_SLOT: float
DEFAULT_TICKS_PER_SECOND: int
DEFAULT_TICKS_PER_SLOT: int
FORWARD_TRANSACTIONS_TO_LEADER_AT_SLOT_OFFSET: int
GENESIS_EPOCH: int
HOLD_TRANSACTIONS_SLOT_OFFSET: int
INITIAL_RENT_EPOCH: int
MAX_HASH_AGE_IN_SECONDS: int
MAX_PROCESSING_AGE: int
MAX_RECENT_BLOCKHASHES: int
MAX_TRANSACTION_FORWARDING_DELAY: int
MAX_TRANSACTION_FORWARDING_DELAY_GPU: int
MS_PER_TICK: int
NUM_CONSECUTIVE_LEADER_SLOTS: int
SECONDS_PER_DAY: int
TICKS_PER_DAY: int

class Clock:
    def __init__(
        self,
        slot: int,
        epoch_start_timestamp: int,
        epoch: int,
        leader_schedule_epoch: int,
        unix_timestamp: int,
    ) -> None: ...
    @property
    def epoch(self) -> int: ...
    @epoch.setter
    def epoch(self, epoch: int) -> None: ...
    @property
    def epoch_start_timestamp(self) -> int: ...
    @epoch_start_timestamp.setter
    def epoch_start_timestamp(self, timestamp: int) -> None: ...
    @property
    def slot(self) -> int: ...
    @slot.setter
    def slot(self, slot: int) -> None: ...
    @property
    def leader_schedule_epoch(self) -> int: ...
    @leader_schedule_epoch.setter
    def leader_schedule_epoch(self, epoch: int) -> None: ...
    @property
    def unix_timestamp(self) -> int: ...
    @unix_timestamp.setter
    def unix_timestamp(self, timestamp: int) -> None: ...
    @staticmethod
    def from_bytes(raw_bytes: bytes) -> "Clock": ...
    @staticmethod
    def from_json(raw: str) -> "Clock": ...
    def to_json(self) -> str: ...
    def __bytes__(self) -> bytes: ...
    def __str__(self) -> str: ...
    def __repr__(self) -> str: ...
    def __richcmp__(self, other: "Clock", op: int) -> bool: ...



from __future__ import annotations

from typing import Optional

class EpochInfo:
    epoch: int
    slot_index: int
    slots_in_epoch: int
    absolute_slot: int
    block_height: int
    transaction_count: Optional[int]
    def __init__(
        self,
        epoch: int,
        slot_index: int,
        slots_in_epoch: int,
        absolute_slot: int,
        block_height: int,
        transaction_count: Optional[int] = None,
    ) -> None: ...
    def to_json(self) -> str: ...
    @staticmethod
    def from_json(raw: str) -> EpochInfo: ...
    @staticmethod
    def from_bytes(data: bytes) -> EpochInfo: ...
    def __str__(self) -> str: ...
    def __repr__(self) -> str: ...
    def __eq__(self, o: object) -> bool: ...
    def __bytes__(self) -> bytes: ...
    def __hash__(self) -> int: ...



from typing import Optional

from solders.pubkey import Pubkey

def get_associated_token_address(
    wallet_address: Pubkey,
    token_mint_address: Pubkey,
    token_program_id: Optional[Pubkey] = None,
) -> Pubkey: ...



from typing import List, Sequence

class SlotHistoryCheck:
    Future: "SlotHistoryCheck"
    TooOld: "SlotHistoryCheck"
    Found: "SlotHistoryCheck"
    NotFound: "SlotHistoryCheck"
    def __int__(self) -> int: ...
    def __str__(self) -> str: ...
    def __repr__(self) -> str: ...
    def __eq__(self, o: object) -> bool: ...

class SlotHistory:
    def __init__(self, bits: Sequence[int], next_slot: int) -> None: ...
    @staticmethod
    def default() -> "SlotHistory": ...
    @property
    def bits(self) -> List[int]: ...
    @bits.setter
    def bits(self, val: Sequence[int]) -> None: ...
    @property
    def next_slot(self) -> int: ...
    @next_slot.setter
    def next_slot(self, val: int) -> None: ...
    def add(self, slot: int) -> None: ...
    def check(self, slot: int) -> SlotHistoryCheck: ...
    def oldest(self) -> int: ...
    def newest(self) -> int: ...
    @staticmethod
    def from_bytes(raw_bytes: bytes) -> "SlotHistory": ...
    @staticmethod
    def from_json(raw: str) -> "SlotHistory": ...
    def to_json(self) -> str: ...
    def __bytes__(self) -> bytes: ...
    def __str__(self) -> str: ...
    def __repr__(self) -> str: ...
    def __richcmp__(self, other: "SlotHistory", op: int) -> bool: ...



from pathlib import Path
from typing import Dict, List, Optional, Sequence, Set, Tuple, Union

from solders.account import Account
from solders.clock import Clock
from solders.compute_budget import ComputeBudget
from solders.epoch_rewards import EpochRewards
from solders.epoch_schedule import EpochSchedule
from solders.hash import Hash
from solders.pubkey import Pubkey
from solders.rent import Rent
from solders.signature import Signature
from solders.slot_history import SlotHistory
from solders.stake_history import StakeHistory
from solders.transaction import Transaction, VersionedTransaction
from solders.transaction_metadata import SimulateResult, TransactionResult

class FeatureSet:
    def __init__(self, active: Dict[Pubkey, int], inactive: Set[Pubkey]) -> None: ...
    @property
    def active(self) -> Dict[Pubkey, int]: ...
    @active.setter
    def active(self, val: Dict[Pubkey, int]) -> None: ...
    @property
    def inactive(self) -> Set[Pubkey]: ...
    @inactive.setter
    def inactive(self, val: Set[Pubkey]) -> None: ...
    @staticmethod
    def default() -> "FeatureSet": ...
    @staticmethod
    def all_enabled() -> "FeatureSet": ...
    def is_active(self, feature_id: Pubkey) -> bool: ...
    def activated_slot(self, feature_id: Pubkey) -> Optional[int]: ...
    def __str__(self) -> str: ...
    def __repr__(self) -> str: ...
    def __richcmp__(self, other: "FeatureSet", op: int) -> bool: ...

class LiteSVM:
    def __init__(self) -> None: ...
    @staticmethod
    def default() -> "LiteSVM": ...
    def set_compute_budget(self, budget: ComputeBudget) -> None: ...
    def set_sigverify(self, sigverify: bool) -> None: ...
    def set_blockhash_check(self, check: bool) -> None: ...
    def set_sysvars(self) -> None: ...
    def set_builtins(self, feature_set: Optional[FeatureSet]) -> None: ...
    def set_lamports(self, lamports: int) -> None: ...
    def set_spl_programs(self) -> None: ...
    def set_transaction_history(self, capacity: int) -> None: ...
    def set_log_bytes_limit(self, limit: Optional[int]) -> None: ...
    def set_precompiles(self, feature_set: Optional[FeatureSet]) -> None: ...
    def minimum_balance_for_rent_exemption(self, data_len: int) -> int: ...
    def get_account(self, pubkey: Pubkey) -> Optional[Account]: ...
    def set_account(self, pubkey: Pubkey, data: Account) -> None: ...
    def get_balance(self, pubkey: Pubkey) -> Optional[int]: ...
    def latest_blockhash(self) -> Hash: ...
    def get_transaction(self, signature: Signature) -> Optional[TransactionResult]: ...
    def airdrop(self, pubkey: Pubkey, lamports: int) -> TransactionResult: ...
    def add_program_from_file(self, program_id: Pubkey, path: Path) -> None: ...
    def add_program(
        self, program_id: Pubkey, program_bytes: Union[bytes, Sequence[int]]
    ) -> None: ...
    def send_transaction(
        self, tx: Union[Transaction, VersionedTransaction]
    ) -> TransactionResult: ...
    def simulate_transaction(
        self, tx: Union[Transaction, VersionedTransaction]
    ) -> SimulateResult: ...
    def expire_blockhash(self) -> None: ...
    def warp_to_slot(self, slot: int) -> None: ...
    def get_compute_budget(self) -> Optional[ComputeBudget]: ...
    def get_clock(self) -> Clock: ...
    def set_clock(self, clock: Clock) -> None: ...
    def get_rent(self) -> Rent: ...
    def set_rent(self, rent: Rent) -> None: ...
    def get_epoch_rewards(self) -> EpochRewards: ...
    def set_epoch_rewards(self, rewards: EpochRewards) -> None: ...
    def get_epoch_schedule(self) -> EpochSchedule: ...
    def set_epoch_schedule(self, schedule: EpochSchedule) -> None: ...
    def get_last_restart_slot(self) -> int: ...
    def set_last_restart_slot(self, slot: int) -> None: ...
    def get_slot_hashes(self) -> List[Tuple[int, Hash]]: ...
    def set_slot_hashes(self, hashes: Sequence[Tuple[int, Hash]]) -> None: ...
    def get_slot_history(self) -> SlotHistory: ...
    def set_slot_history(self, history: SlotHistory) -> None: ...
    def get_stake_history(self) -> StakeHistory: ...
    def set_stake_history(self, history: StakeHistory) -> None: ...



import contextlib as __ctxlib

from solders.solders import (  # type: ignore
    address_lookup_table_account,
    clock,
    commitment_config,
    compute_budget,
    epoch_info,
    epoch_rewards,
    epoch_schedule,
    errors,
    hash,
    instruction,
    keypair,
    message,
    null_signer,
    presigner,
    pubkey,
    rent,
    signature,
    slot_history,
    stake_history,
    token,
    transaction,
)

from . import system_program, sysvar

__has_bankrun = False
with __ctxlib.suppress(ImportError):
    from solders.solders import bankrun

    __has_bankrun = True

__has_ring = False
with __ctxlib.suppress(ImportError):
    from solders.solders import account, account_decoder, rpc, transaction_status

    __has_ring = True


__ring_modules = ["account", "account_decoder", "rpc", "transaction_status"]

__all_core = [
    "address_lookup_table_account",
    "commitment_config",
    "errors",
    "hash",
    "instruction",
    "keypair",
    "message",
    "null_signer",
    "presigner",
    "pubkey",
    "signature",
    "token",
    "transaction",
    "system_program",
    "sysvar",
]

__with_ring_modules = [*__all_core, *__ring_modules]

if __has_ring:
    if __has_bankrun:
        __all__ = [*__with_ring_modules, "bankrun"]  # noqa: PLE0604
    else:
        __all__ = __with_ring_modules  # noqa: PLE0605
else:
    __all__ = __all_core  # noqa: PLE0605


__version__ = "0.24.0"



from typing import List, Optional, Sequence, Tuple, Union

from typing_extensions import Final

from solders.hash import Hash
from solders.pubkey import Pubkey

ID: Final[Pubkey]
LOOKUP_TABLE_MAX_ADDRESSES: Final[int]
LOOKUP_TABLE_META_SIZE: Final[int]

class AddressLookupTableAccount:
    def __init__(
        self,
        key: Pubkey,
        addresses: Sequence[Pubkey],
    ) -> None: ...
    def __bytes__(self) -> bytes: ...
    def __str__(self) -> str: ...
    def __repr__(self) -> str: ...
    def __richcmp__(self, other: "AddressLookupTableAccount", op: int) -> bool: ...
    @staticmethod
    def from_bytes(raw_bytes: bytes) -> "AddressLookupTableAccount": ...
    def to_json(self) -> str: ...
    @staticmethod
    def from_json(raw: str) -> "AddressLookupTableAccount": ...
    @property
    def key(self) -> Pubkey: ...
    @property
    def addresses(self) -> List[Pubkey]: ...

class AddressLookupTable:
    def __init__(self, key: Pubkey, addresses: Sequence[Pubkey]) -> None: ...
    def get_active_addresses_len(
        self, current_slot: int, slot_hashes: SlotHashes
    ) -> int: ...
    def lookup(
        self,
        current_slot: int,
        indexes: Sequence[int],
        slot_hashes: SlotHashes,
    ) -> List[Pubkey]: ...
    @staticmethod
    def deserialize(data: bytes) -> AddressLookupTable: ...
    @property
    def meta(self) -> LookupTableMeta: ...
    @property
    def addresses(self) -> List[Pubkey]: ...
    def __bytes__(self) -> bytes: ...
    def __str__(self) -> str: ...
    def __repr__(self) -> str: ...
    def __richcmp__(self, other: "AddressLookupTable", op: int) -> bool: ...
    @staticmethod
    def from_bytes(raw_bytes: bytes) -> "AddressLookupTable": ...
    def to_json(self) -> str: ...
    @staticmethod
    def from_json(raw: str) -> "AddressLookupTable": ...

class LookupTableMeta:
    def __init__(
        self,
        deactivation_slot: int = 18446744073709551615,  # u64::MAX
        last_extended_slot: int = 0,
        last_extended_slot_start_index: int = 0,
        authority: Optional[Pubkey] = None,
        padding: int = 0,
    ) -> None: ...
    def is_active(self, current_slot: int, slot_hashes: SlotHashes) -> bool: ...
    def status(
        self, current_slot: int, slot_hashes: SlotHashes
    ) -> LookupTableStatusType: ...
    @property
    def deactivation_slot(self) -> int: ...
    @property
    def last_extended_slot(self) -> int: ...
    @property
    def last_extended_slot_start_index(self) -> int: ...
    @property
    def authority(self) -> Optional[Pubkey]: ...
    @property
    def padding(self) -> int: ...
    def __bytes__(self) -> bytes: ...
    def __str__(self) -> str: ...
    def __repr__(self) -> str: ...
    def __richcmp__(self, other: "LookupTableMeta", op: int) -> bool: ...
    @staticmethod
    def from_bytes(raw_bytes: bytes) -> "LookupTableMeta": ...
    def to_json(self) -> str: ...
    @staticmethod
    def from_json(raw: str) -> "LookupTableMeta": ...

class LookupTableStatusFieldless:
    Activated: "LookupTableStatusFieldless"
    Deactivated: "LookupTableStatusFieldless"
    def __int__(self) -> int: ...
    def __str__(self) -> str: ...
    def __repr__(self) -> str: ...
    def __eq__(self, o: object) -> bool: ...

class LookupTableStatusDeactivating:
    def __init__(self, remaining_slots: int) -> None: ...
    @property
    def remaining_slots(self) -> int: ...
    def __bytes__(self) -> bytes: ...
    def __str__(self) -> str: ...
    def __repr__(self) -> str: ...
    def __richcmp__(self, other: "LookupTableStatusDeactivating", op: int) -> bool: ...
    @staticmethod
    def from_bytes(raw_bytes: bytes) -> "LookupTableStatusDeactivating": ...
    def to_json(self) -> str: ...
    @staticmethod
    def from_json(raw: str) -> "LookupTableStatusDeactivating": ...

class SlotHashes:
    def __init__(self, slot_hashes: Sequence[Tuple[int, Hash]]) -> None: ...
    @property
    def slot_hashes(self) -> List[Tuple[int, Hash]]: ...
    def __bytes__(self) -> bytes: ...
    def __str__(self) -> str: ...
    def __repr__(self) -> str: ...
    def __richcmp__(self, other: "SlotHashes", op: int) -> bool: ...
    @staticmethod
    def from_bytes(raw_bytes: bytes) -> "SlotHashes": ...
    def to_json(self) -> str: ...
    @staticmethod
    def from_json(raw: str) -> "SlotHashes": ...

LookupTableStatusType = Union[LookupTableStatusFieldless, LookupTableStatusDeactivating]

def derive_lookup_table_address(
    authority_address: Pubkey, recent_block_slot: int
) -> Tuple[Pubkey, int]: ...



from solders.instruction import Instruction
from solders.pubkey import Pubkey

ID: Pubkey

def request_heap_frame(bytes_: int) -> Instruction: ...
def set_compute_unit_limit(units: int) -> Instruction: ...
def set_compute_unit_price(micro_lamports: int) -> Instruction: ...

class ComputeBudget:
    def __init__(self) -> None: ...
    @property
    def compute_unit_limit(self) -> int: ...
    @compute_unit_limit.setter
    def compute_unit_limit(self, val: int) -> None: ...
    @property
    def log_64_units(self) -> int: ...
    @log_64_units.setter
    def log_64_units(self, val: int) -> None: ...
    @property
    def create_program_address_units(self) -> int: ...
    @create_program_address_units.setter
    def create_program_address_units(self, val: int) -> None: ...
    @property
    def invoke_units(self) -> int: ...
    @invoke_units.setter
    def invoke_units(self, val: int) -> None: ...
    @property
    def max_instruction_stack_depth(self) -> int: ...
    @max_instruction_stack_depth.setter
    def max_instruction_stack_depth(self, val: int) -> None: ...
    @property
    def max_instruction_trace_length(self) -> int: ...
    @max_instruction_trace_length.setter
    def max_instruction_trace_length(self, val: int) -> None: ...
    @property
    def sha256_base_cost(self) -> int: ...
    @sha256_base_cost.setter
    def sha256_base_cost(self, val: int) -> None: ...
    @property
    def sha256_byte_cost(self) -> int: ...
    @sha256_byte_cost.setter
    def sha256_byte_cost(self, val: int) -> None: ...
    @property
    def sha256_max_slices(self) -> int: ...
    @sha256_max_slices.setter
    def sha256_max_slices(self, val: int) -> None: ...
    @property
    def max_call_depth(self) -> int: ...
    @max_call_depth.setter
    def max_call_depth(self, val: int) -> None: ...
    @property
    def stack_frame_size(self) -> int: ...
    @stack_frame_size.setter
    def stack_frame_size(self, val: int) -> None: ...
    @property
    def log_pubkey_units(self) -> int: ...
    @log_pubkey_units.setter
    def log_pubkey_units(self, val: int) -> None: ...
    @property
    def max_cpi_instruction_size(self) -> int: ...
    @max_cpi_instruction_size.setter
    def max_cpi_instruction_size(self, val: int) -> None: ...
    @property
    def cpi_bytes_per_unit(self) -> int: ...
    @cpi_bytes_per_unit.setter
    def cpi_bytes_per_unit(self, val: int) -> None: ...
    @property
    def sysvar_base_cost(self) -> int: ...
    @sysvar_base_cost.setter
    def sysvar_base_cost(self, val: int) -> None: ...
    @property
    def secp256k1_recover_cost(self) -> int: ...
    @secp256k1_recover_cost.setter
    def secp256k1_recover_cost(self, val: int) -> None: ...
    @property
    def syscall_base_cost(self) -> int: ...
    @syscall_base_cost.setter
    def syscall_base_cost(self, val: int) -> None: ...
    @property
    def curve25519_edwards_validate_point_cost(self) -> int: ...
    @curve25519_edwards_validate_point_cost.setter
    def curve25519_edwards_validate_point_cost(self, val: int) -> None: ...
    @property
    def curve25519_edwards_add_cost(self) -> int: ...
    @curve25519_edwards_add_cost.setter
    def curve25519_edwards_add_cost(self, val: int) -> None: ...
    @property
    def curve25519_edwards_subtract_cost(self) -> int: ...
    @curve25519_edwards_subtract_cost.setter
    def curve25519_edwards_subtract_cost(self, val: int) -> None: ...
    @property
    def curve25519_edwards_multiply_cost(self) -> int: ...
    @curve25519_edwards_multiply_cost.setter
    def curve25519_edwards_multiply_cost(self, val: int) -> None: ...
    @property
    def curve25519_edwards_msm_base_cost(self) -> int: ...
    @curve25519_edwards_msm_base_cost.setter
    def curve25519_edwards_msm_base_cost(self, val: int) -> None: ...
    @property
    def curve25519_edwards_msm_incremental_cost(self) -> int: ...
    @curve25519_edwards_msm_incremental_cost.setter
    def curve25519_edwards_msm_incremental_cost(self, val: int) -> None: ...
    @property
    def curve25519_ristretto_validate_point_cost(self) -> int: ...
    @curve25519_ristretto_validate_point_cost.setter
    def curve25519_ristretto_validate_point_cost(self, val: int) -> None: ...
    @property
    def curve25519_ristretto_add_cost(self) -> int: ...
    @curve25519_ristretto_add_cost.setter
    def curve25519_ristretto_add_cost(self, val: int) -> None: ...
    @property
    def curve25519_ristretto_subtract_cost(self) -> int: ...
    @curve25519_ristretto_subtract_cost.setter
    def curve25519_ristretto_subtract_cost(self, val: int) -> None: ...
    @property
    def curve25519_ristretto_multiply_cost(self) -> int: ...
    @curve25519_ristretto_multiply_cost.setter
    def curve25519_ristretto_multiply_cost(self, val: int) -> None: ...
    @property
    def curve25519_ristretto_msm_base_cost(self) -> int: ...
    @curve25519_ristretto_msm_base_cost.setter
    def curve25519_ristretto_msm_base_cost(self, val: int) -> None: ...
    @property
    def curve25519_ristretto_msm_incremental_cost(self) -> int: ...
    @curve25519_ristretto_msm_incremental_cost.setter
    def curve25519_ristretto_msm_incremental_cost(self, val: int) -> None: ...
    @property
    def heap_size(self) -> int: ...
    @heap_size.setter
    def heap_size(self, val: int) -> None: ...
    @property
    def heap_cost(self) -> int: ...
    @heap_cost.setter
    def heap_cost(self, val: int) -> None: ...
    @property
    def mem_op_base_cost(self) -> int: ...
    @mem_op_base_cost.setter
    def mem_op_base_cost(self, val: int) -> None: ...
    @property
    def alt_bn128_addition_cost(self) -> int: ...
    @alt_bn128_addition_cost.setter
    def alt_bn128_addition_cost(self, val: int) -> None: ...
    @property
    def alt_bn128_multiplication_cost(self) -> int: ...
    @alt_bn128_multiplication_cost.setter
    def alt_bn128_multiplication_cost(self, val: int) -> None: ...
    @property
    def alt_bn128_pairing_one_pair_cost_first(self) -> int: ...
    @alt_bn128_pairing_one_pair_cost_first.setter
    def alt_bn128_pairing_one_pair_cost_first(self, val: int) -> None: ...
    @property
    def alt_bn128_pairing_one_pair_cost_other(self) -> int: ...
    @alt_bn128_pairing_one_pair_cost_other.setter
    def alt_bn128_pairing_one_pair_cost_other(self, val: int) -> None: ...
    @property
    def big_modular_exponentiation_base_cost(self) -> int: ...
    @big_modular_exponentiation_base_cost.setter
    def big_modular_exponentiation_base_cost(self, val: int) -> None: ...
    @property
    def big_modular_exponentiation_cost_divisor(self) -> int: ...
    @big_modular_exponentiation_cost_divisor.setter
    def big_modular_exponentiation_cost_divisor(self, val: int) -> None: ...
    @property
    def poseidon_cost_coefficient_a(self) -> int: ...
    @poseidon_cost_coefficient_a.setter
    def poseidon_cost_coefficient_a(self, val: int) -> None: ...
    @property
    def poseidon_cost_coefficient_c(self) -> int: ...
    @poseidon_cost_coefficient_c.setter
    def poseidon_cost_coefficient_c(self, val: int) -> None: ...
    @property
    def remaining_compute_units_cost(self) -> int: ...
    @remaining_compute_units_cost.setter
    def remaining_compute_units_cost(self, val: int) -> None: ...
    @property
    def alt_bn128_g1_compress(self) -> int: ...
    @alt_bn128_g1_compress.setter
    def alt_bn128_g1_compress(self, val: int) -> None: ...
    @property
    def alt_bn128_g1_decompress(self) -> int: ...
    @alt_bn128_g1_decompress.setter
    def alt_bn128_g1_decompress(self, val: int) -> None: ...
    @property
    def alt_bn128_g2_compress(self) -> int: ...
    @alt_bn128_g2_compress.setter
    def alt_bn128_g2_compress(self, val: int) -> None: ...
    @property
    def alt_bn128_g2_decompress(self) -> int: ...
    @alt_bn128_g2_decompress.setter
    def alt_bn128_g2_decompress(self, val: int) -> None: ...
    def __str__(self) -> str: ...
    def __repr__(self) -> str: ...
    def __richcmp__(self, other: "ComputeBudget", op: int) -> bool: ...



from typing import List, Optional, Sequence

from solders.pubkey import Pubkey

class Mint:
    decimals: int
    freeze_authority: Optional[Pubkey]
    is_initialized: bool
    mint_authority: Optional[Pubkey]
    supply: int
    @classmethod
    def __init__(
        self,
        mint_authority: Optional[Pubkey],
        supply: int,
        decimals: int,
        is_initialized: bool,
        freeze_authority: Optional[Pubkey] = None,
    ) -> None: ...
    @staticmethod
    def default() -> "Mint": ...
    @staticmethod
    def from_bytes(raw: bytes) -> "Mint": ...
    def __bytes__(self) -> bytes: ...
    def __richcmp__(self, other: "Mint", op: int) -> bool: ...
    def __str__(self) -> str: ...
    def __repr__(self) -> str: ...

class Multisig:
    is_initialized: bool
    m: int
    n: int
    signers: List[Pubkey]
    @classmethod
    def __init__(
        self, m: int, n: int, is_initialized: bool, signers: Sequence[Pubkey]
    ) -> None: ...
    @staticmethod
    def default() -> "Multisig": ...
    @staticmethod
    def from_bytes(raw: bytes) -> "Multisig": ...
    def __richcmp__(self, other: "Multisig", op: int) -> bool: ...
    def __bytes__(self) -> bytes: ...
    def __str__(self) -> str: ...
    def __repr__(self) -> str: ...

class TokenAccountState:
    Uninitialized: "TokenAccountState"
    Initialized: "TokenAccountState"
    Frozen: "TokenAccountState"
    def __int__(self) -> int: ...
    def __str__(self) -> str: ...
    def __repr__(self) -> str: ...
    def __eq__(self, o: object) -> bool: ...

class TokenAccount:
    amount: int
    close_authority: Optional[Pubkey]
    delegate: Optional[Pubkey]
    delegated_amount: int
    is_native: Optional[int]
    mint: Pubkey
    owner: Pubkey
    state: TokenAccountState
    def __init__(
        self,
        mint: Pubkey,
        owner: Pubkey,
        amount: int,
        delegate: Optional[Pubkey],
        state: TokenAccountState,
        is_native: Optional[int],
        delegated_amount: int,
        close_authority: Optional[int] = None,
    ) -> None: ...
    @staticmethod
    def default() -> "TokenAccount": ...
    @staticmethod
    def from_bytes(raw: bytes) -> "TokenAccount": ...
    def __richcmp__(self, other: "TokenAccount", op: int) -> bool: ...
    def __bytes__(self) -> bytes: ...
    def __str__(self) -> str: ...
    def __repr__(self) -> str: ...



from typing import Optional, Tuple

ACCOUNT_STORAGE_OVERHEAD: int
DEFAULT_BURN_PERCENT: int
DEFAULT_EXEMPTION_THRESHOLD: float
DEFAULT_LAMPORTS_PER_BYTE_YEAR: int

class Rent:
    @property
    def burn_percent(self) -> int: ...
    @property
    def exemption_threshold(self) -> float: ...
    @property
    def lamports_per_byte_year(self) -> int: ...
    def __init__(
        self, burn_percent: int, exemption_threshold: float, lamports_per_byte_year: int
    ) -> None: ...
    def calculate_burn(self, rent_collected: int) -> Tuple[int, int]: ...
    @staticmethod
    def default() -> "Rent": ...
    def due(
        self, balance: int, data_len: int, years_elapsed: float
    ) -> Optional[int]: ...
    def due_amount(self, data_len: int, years_elapsed: float) -> int: ...
    @staticmethod
    def free() -> "Rent": ...
    @staticmethod
    def from_bytes(raw_bytes: bytes) -> "Rent": ...
    @staticmethod
    def from_json(raw: str) -> "Rent": ...
    def is_exempt(self, balance: int, data_len: int) -> bool: ...
    def minimum_balance(self, data_len: int) -> int: ...
    def to_json(self) -> str: ...
    def with_slots_per_epoch(self, slots_per_epoch: int) -> "Rent": ...
    def __bytes__(self) -> bytes: ...
    def __str__(self) -> str: ...
    def __repr__(self) -> str: ...
    def __richcmp__(self, other: "Rent", op: int) -> bool: ...
